'' =================================================================================================
''
''   File....... hc-8+_vixen_streamer_2013.spin
''   Purpose.... Receives Vixen data stream, converts to output
''               -- allows combination of servos and dimmers
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2011-2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com 
''   Started.... 
''   Updated.... 20 AUG 2013
''
'' =================================================================================================

{{

  This program receives a stream of channel values from Vixen through the programming
  port of the HC-8+ (using a Prop-Plug). 

  In Vixen:
    -- assign Generic Serial plug-in to project
       * 57.6K baud
       * no header
       * no footer
    -- set event period based on type of project
       * 20ms for smooth LED fading / servo control

  The Vixen project must be setup to transmit 8 channels; the first four channels are
  interpreted as servos, the second four as dimmer/valve outputs.

  Servos are connected to the TTL header just above the option swtich.

  outs, lamps, valves, relays, etc., are connected to green terminals blocks.      

}}


con

  PCB_REV = "C"


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000
  

con

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom
  SCL     = 28

  OUT_EN  = 27                                                  ' outputs enable pin (low)     
  
  SIO     = 26                                                  ' TTL serial io
  
  RX2     = 23                                                  ' RS-485 port
  RX2_EN  = 22
  TX2_EN  = 21
  TX2     = 20

  R_LED   = 18                                                  ' bi-color LED
  G_LED   = 17

  OUT7    = 15                                                  ' outputs
  OUT6    = 14   
  OUT5    = 13 
  OUT4    = 12
  OUT3    = 11
  OUT2    = 10   
  OUT1    =  9 
  OUT0    =  8
          
  OPT_BR  =  7                                                  ' options swithes
  OPT_A1  =  6   
  OPT_A0  =  5
  OPT_SM  =  4

  DMX_A8  =  3                                                  ' high-bit off DMX address

  DO_165  =  2                                                  ' control lines for 74x165
  CLK_165 =  1 
  LD_165  =  0


  ' for EFX-TEK uSD adapter module (#99002)
  ' -- must open all option switches when installed

  SD_CS   =  OPT_BR                                       
  SD_DI   =  OPT_A1  
  SD_CLK  =  OPT_A0  
  SD_DO   =  OPT_SM


con

   #0, OFF, RED, GRN, YEL                                       ' for r/g led

   #0, NOCON, RX, TX, RXTX                                      ' for rs-485


con

  BAUD     = 57_600                                             ' for Vixen serial driver
  

  ' Note: Servo channels are on the TTL header above the option switch
  '       With 4 servos use positions 0..3.
  
  SERVOS   = 4                                                  ' servo channels
  SVO_BASE = OUT0                                               ' servos start at OUT0 (TTL)

  SVO_MIN  = 0_600                                              ' 180-degree servos
  SVO_MAX  = 2_400
  
  
  ' Note: Dimmer channels follow servo channels. With 4 servos there will
  '       be 4 dimmer channels, use OUT4..OUT7 (terminal blocks)
  
  DIMMERS  = 8 - SERVOS                                         ' rest are dimmer channels
  DIM_BASE = OUT0 + SERVOS                                      ' dimmers follow servos


obj

  serial : "fullduplexserial64"                                 ' for input from Vixen
  svos   : "jm_servo8"                                          ' servo driver
  outs   : "jm_pwm8"                                            ' pwm driver for LED dimmers
       

var


dat

' Starting positions for servos when driver is loaded
' -- provide data for 8 servos to be safe

ServoStart              long    1_500, 1_500, 1_500, 1_500
                        long    1_500, 1_500, 1_500, 1_500


pub main | ch, pos, level

  setup_io                                                      ' configure HC-8 io pins

  serial.start(RX1, TX1, %0000, BAUD)                           ' start serial driver

  if (SERVOS > 0)                                               ' start servo driver
    svos.startx2(SERVOS, SVO_BASE, SVO_MIN, SVO_MAX, @ServoStart, $FF)

  if (DIMMERS > 0)                                              ' start pwm driver
    outs.start(DIMMERS, DIM_BASE)

  pause(100)
  serial.rxflush                                                ' clean-up serial input

  repeat { main loop }
    repeat                                                      ' wait for idle state
      level := serial.rxtime(5)                                 ' -- at least 5ms
    until (level < 0)

    ' wait for start of packet

    if (SERVOS > 0)
      pos := serial.rx                                          ' get raw value from Vixen                        
      pos := dmx_scale(pos, SVO_MIN, SVO_MAX)                   ' convert to microseconds
      svos.set(0, pos, 0)                                       ' set servo
    else
      level := serial.rx                                        ' get raw value
      outs.set(0, level)                                        ' output as pwm

    set_rgled(GRN)                                              ' show packet arrived

    ' get rest of packet
    
    repeat ch from 1 to 7
      if (ch < SERVOS)                                          ' more servos?
        pos := serial.rxtime(1)                                 ' get position
        if (pos => 0)                                           ' no timeout?
          pos := dmx_scale(pos, SVO_MIN, SVO_MAX)               ' -- scale and set servo
          svos.set(ch, pos, 0)
        else
          quit                                                  ' timeout! abort this packet
      else { dimmers }
        level := serial.rxtime(1)
        if (level => 0)    
          outs.set(ch, level)
        else
          quit

    set_rgled(OFF)                                              ' end of packet         


pub dmx_scale(in, outmin, outmax) | scale

'' Scale DMX-compatible value (0..255) to outmin..outmax

  if (in =< 0)
    scale := outmin
  elseif (in => 255)
    scale := outmax 
  else
    scale := (outmax - outmin) * in / 255 + outmin
    
  return scale


con

  { --------------------------------- }
  {  S U P P O R T   R O U T I N E S  } 
  { --------------------------------- }


pub setup_io

'' Configure for basic IO control

  outa[OUT7..OUT0] := %0000_0000                                ' clear outputs
  dira[OUT7..OUT0] := %1111_1111                                ' output mode                                  
  low(OUT_EN)                                                   ' enable 74x245 

  high(LD_165)                                                  ' setup x165 io pins
  low(CLK_165)
  input(DO_165)
  
  set_rs485(NOCON)                                              ' RS-485 off


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be at least 1
    return
  else
    t := cnt - 1792                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)


pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles output pin

  !outa[pin]                                                    ' invert state
  dira[pin] := 1  
  

pub input(pin)

  dira[pin] := 0

  return ina[pin]


pub set_rgled(state)

'' Sets R/G LED
'' -- direct control only
'' -- for red/green/yellow use "efx_rgled" driver and methods

  case state
    OFF : outa[R_LED..G_LED] := %00
    RED : outa[R_LED..G_LED] := %10
    GRN : outa[R_LED..G_LED] := %01
    YEL : outa[R_LED..G_LED] := %00                             ' yellow requires PASM driver

  dira[R_LED..G_LED] := %11
      

pub set_rs485(state)

'' Sets RS-485 RX and TX enable inputs

  case state
    NOCON : outa[RX2_EN..TX2_EN] := %10
    RX    : outa[RX2_EN..TX2_EN] := %00 
    TX    : outa[RX2_EN..TX2_EN] := %11   
    RXTX  : outa[RX2_EN..TX2_EN] := %01

  dira[RX2_EN..TX2_EN] := %11


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      